
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "assuma_addonengine.h"
#include <QFile>
#include <QDir>

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

#include <QMap>
#include <QString>
#include <QByteArray>
#include <QUiLoader>
#include <QtScript>
#include "addonxmlhandler.h"
#include "assuma_addoninstance.h"

AssumaAddonEngine::AssumaAddonEngine( const QString & addonsDirectory) : QObject(){
	_engines.empty();
	_uis.empty();
	_addonInstances.empty();
	_addonsDirectory = addonsDirectory;
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaAddonEngine::AssumaAddonEngine(" << addonsDirectory << ") -> _addonsDirectory:" << _addonsDirectory;
#endif
}

QList<AssumaAddonInstance*> AssumaAddonEngine::instanciatedAddons(){
	return _addonInstances.values();
}

AssumaAddonInstance AssumaAddonEngine::loadAddon(const QString & addonName){
	QFile xml_addon_file( _addonsDirectory + "/" + addonName + "/addon.xml" );
	if( ! xml_addon_file.exists() )
		return AssumaAddonInstance();
	QXmlInputSource inputSource(&xml_addon_file);
	QXmlSimpleReader reader;
	AddonXmlHandler *handler = new AddonXmlHandler();
	reader.setContentHandler(handler);
	reader.setErrorHandler(handler);
	reader.parse(inputSource);
	QMap<QString,QString> data = handler->getData();
	AssumaAddonInstance tmp_addon;
	if( validateAddonData(data) ){
		foreach( QString key, data.keys() ){
			foreach( QString tmp_value, data.values(key) ){
#ifdef ASSUMA_DEBUG
				qDebug() << "AssumaAddonEngine::loadAddon(" << addonName << ") inserting into AssumaAddonInstance: " << key << "=>" << tmp_value;
#endif
				tmp_addon.insertValue( key, tmp_value );
			}
		}
		return tmp_addon;
	}
	else
		return AssumaAddonInstance();
}

bool AssumaAddonEngine::instanciateAddon( const QString & addonName ){
	QScriptEngine tmp_engine ;
	QFile xml_addon_file( _addonsDirectory + "/" + addonName + "/addon.xml" );
	QXmlInputSource inputSource(&xml_addon_file);
	QXmlSimpleReader reader;
	AddonXmlHandler *handler = new AddonXmlHandler();
	reader.setContentHandler(handler);
	reader.setErrorHandler(handler);
	reader.parse(inputSource);
	QMap<QString,QString> data = handler->getData();
	AssumaAddonInstance *tmp_addon = new AssumaAddonInstance();
	if( validateAddonData(data) ){
		foreach( QString key, data.keys() ){
			foreach( QString tmp_value, data.values(key) ){
#ifdef ASSUMA_DEBUG
				qDebug() << "AssumaAddonEngine::instanciateAddon(" << addonName << ") inserting into AssumaAddonInstance: " << key << "=>" << tmp_value;
#endif
				tmp_addon->insertValue( key, tmp_value );
			}
		}
		if( tmp_addon->name() == addonName ){
			_addonInstances.insert(addonName,tmp_addon);
			// Check if addon is a script type.
			if( tmp_addon->type() == "script" ){
				// Create the script engine and a temp bytearray
				_engines.insert( addonName, new QScriptEngine() );
				QScriptEngine *engine = _engines.value(addonName);
				QByteArray ba;
				// Get all code files and concatenate them.
				foreach (QString tmp_codeFile, tmp_addon->codeFiles() ){
					QFile scriptFile( _addonsDirectory + "/" + addonName + "/" + tmp_codeFile );
					scriptFile.open(QIODevice::ReadOnly);
					ba.append( scriptFile.readAll() );
					scriptFile.close();
				}
#ifdef ASSUMA_DEBUG
				qDebug() << "AssumaAddonEngine::instanciateAllAddons() : evaluating QByteArray :" << ba;
#endif
				engine->evaluate( ba );
				//_engines.insert( addonName , engine );
				
				if( ! tmp_addon->uiFile().isEmpty() ) {
					// Now creating the UI
					QUiLoader loader;
					QFile uiFile( _addonsDirectory + "/" + addonName + "/" + tmp_addon->uiFile() );
					uiFile.open(QIODevice::ReadOnly);
					QWidget *ui = loader.load(&uiFile);
					uiFile.close();
					_uis.insert( addonName , ui );
					
				}
				// Finally constructing the Addon
				QScriptValue ctor = engine->evaluate( tmp_addon->name() );
				QScriptValue cret;
				if( _uis.value( addonName ) ){
					QScriptValue scriptUi = engine->newQObject(_uis.value( addonName ), QScriptEngine::ScriptOwnership);
					cret = ctor.construct(QScriptValueList() << scriptUi);
// 					_uis.value( addonName )->show();
				}
				else
					cret = ctor.construct( QScriptValueList() );
				
				/*! \todo replace the actual call for the Init() function with a signal */
				if( engine->canEvaluate("Init") ){
					QScriptValue cinit = engine->evaluate("Init");
					cinit.call(cret);
				}
			}
			return true;
		}
		else
			return false;
	}
	else
		return false;
}

void AssumaAddonEngine::instanciateAllAddons(){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaAddonEngine::instanciateAllAddons() -> addons directory is :" << _addonsDirectory;
#endif
	QDir pDir( _addonsDirectory );
	foreach ( QString subDir, pDir.entryList( QDir::Dirs | QDir::NoDotAndDotDot ) ){
		if( instanciateAddon(subDir) ){
#ifdef ASSUMA_DEBUG
			AssumaAddonInstance *tmp_addon = _addonInstances.value( subDir );
			qDebug() << "AssumaAddonEngine::instanciateAllAddons() -> Addon successfully instantiated :" << tmp_addon->name() << "v." << tmp_addon->version() << "by" << tmp_addon->author();
#else
			_addonInstances.value( subDir );
#endif
		}
		else
			emit( addonLoadingFailed( subDir ) );
	}
}

bool AssumaAddonEngine::validateAddonData( const QMap<QString,QString> & data ){
	if( data.value("Name").isEmpty() || data.value("Version").isEmpty() || data.value("Type").isEmpty() || data.values("CodeFile").isEmpty() ){
		return false;
	}
	return true;
}

bool AssumaAddonEngine::destroyAddon( const QString & addonName ){
	if( _uis.value(addonName) ){
		_uis.value(addonName)->close();
		delete _uis.value(addonName);
	}
	delete _engines.value(addonName);
	delete _addonInstances.value(addonName);
	return true;
}

void AssumaAddonEngine::destroyAllAddons(){
	foreach( QString addon , _addonInstances.keys() ){
		destroyAddon( addon );
	}
	_engines.empty();
	_uis.empty();
	_addonInstances.empty();
}

void AssumaAddonEngine::setAddonObjectProperty(const QString& addonName, const QString & propName, QObject * object){
	if( _engines.value(addonName) ){
		QScriptValue objectValue = _engines.value(addonName)->newQObject(object);
		_engines.value(addonName)->globalObject().setProperty(propName, objectValue);
	}
}

void AssumaAddonEngine::setAddonObjectProperty(const QString & propName, QObject * object){
	foreach( QString addon , _engines.keys() ){
		setAddonObjectProperty( addon, propName, object );
	}
}
