#include "assuma_wizard.h"
#include <QSettings>
#include <QFile>
#include <QDir>
#include <QTextStream>
#include <QMessageBox>

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

#include <QFileDialog>
#include <QTranslator>
#include <QDesktopServices>
#include <QUrl>

#ifndef ASSUMA_DATADIR
#define ASSUMA_DATADIR "/usr/share/AssociationSubscribersManager"
#endif

#ifndef ASSUMA_DOCDIR
#define ASSUMA_DOCDIR "/usr/share/AssociationSubscribersManager/doc"
#endif

#ifndef ASSUMA_IMGDIR
#define ASSUMA_IMGDIR "/usr/share/AssociationSubscribersManager/images"
#endif

AssumaWizard::AssumaWizard(QWidget *parent) : QWizard(parent){
	setupUi(this);
	shouldRun=true;
	m_helpBrowser = new AssumaHelpBrowser(this);
	copyDataDirLabel->setVisible(false);
	copyDataDirectoryLineEdit->setVisible(false);
	copyDataDirectoryToolButton->setVisible(false);
	errorLabel->setText("");
	connect( this, SIGNAL( currentIdChanged ( int ) ), this, SLOT(pageChanged(int) )  );
	connect( button(QWizard::FinishButton), SIGNAL(clicked(bool) ),this, SLOT(writeSettings() ) );
	connect( button(QWizard::HelpButton), SIGNAL(clicked(bool) ),this, SLOT(showHelp() ) );
	readSettings();
}

bool AssumaWizard::haveAlreadyRun(){
	return !shouldRun;
}

void AssumaWizard::readSettings(){
	QSettings settings("Infinity Perl","assuma");
	
	// Restoring settings from the configuration widget
	dataDirectoryLineEdit->setText(settings.value("configDataDir", QString(ASSUMA_DATADIR) ).toString());
#ifdef Q_OS_MAC
	QDir _dir = QDir(qApp->applicationDirPath());
	if( _dir.dirName() == "MacOS" ){
		_dir.cdUp();
		_dir.cd("Resources/data");
	}
	dataDirectoryLineEdit->setText(settings.value("configDataDir", _dir.absolutePath() ).toString());
	copyDataDirCheckBox->setCheckState(Qt::Checked);
	copyDataDirectoryLineEdit->setText( QDir::homePath() );
#endif
	// DataDir is set, this is all we need to load the list of available languages.
	loadLanguageList( dataDirectoryLineEdit->text().append("/langpack.lst") );
	
	if( settings.value("configUpdatesOnStart",QString::number(1)).toString() == "1" )
		updateOnStartCheckBox->setCheckState( Qt::Checked );
	
	languageComboBox->setCurrentIndex( languageComboBox->findText(settings.value("configLanguage","English").toString(), Qt::MatchExactly | Qt::MatchCaseSensitive ) );
	assoNameLineEdit->setText( settings.value("configAssociationName","My Association").toString() );
	assoEmailLineEdit->setText( settings.value("configEmail","Owner@MyAssociation.org").toString() );
	smtpLineEdit->setText( settings.value("configSmtpServer","localhost").toString() );
	shouldRun = settings.value("wizardShouldRun",true).toBool();
}

void AssumaWizard::writeSettings(){
	QSettings settings("Infinity Perl","assuma");
	QString updateServer = settings.value("configUpdateServer","http://assuma.sourceforge.net").toString();
	if( updateServer.contains("infinityperl.org") )
		updateServer = "http://assuma.sourceforge.net";
	
	settings.setValue("configUpdateServer", updateServer );
	if( updateOnStartCheckBox->checkState() == Qt::Checked )
		settings.setValue("configUpdatesOnStart", QString::number(1) );
	else
		settings.setValue("configUpdatesOnStart", QString::number(0) );
	settings.setValue("wizardShouldRun",false);
	settings.setValue("configAssociationName", assoNameLineEdit->text() );
	settings.setValue("configLanguage", languageComboBox->currentText() );
	if( languageComboBox->currentText() == "Svenska" ){
		settings.setValue("configCurrency", "svensk krona (SEK)" );
	}
	else if( languageComboBox->currentText() == "English" ){
		settings.setValue("configCurrency", QString::fromUtf8("Pound (£)") );
		settings.setValue("configCountry", "GreatBritain" );
	}
	else {
		settings.setValue("configCurrency", QString::fromUtf8("Euro (€)") );
		if( languageComboBox->currentText().startsWith("Fran") )
			settings.setValue("configCountry", "France" );
	}
	settings.setValue("configDataDir", dataDirectoryLineEdit->text() );
	settings.setValue("configSmtpServer", smtpLineEdit->text() );
	settings.setValue("configEmail", assoEmailLineEdit->text() );
}

void AssumaWizard::loadLanguageList( const QString & lp_list_file ){
	QFile file( lp_list_file );
	languageComboBox->clear();
	if (!file.open(QIODevice::ReadOnly | QIODevice::Text)){
		return ;
	}
	QTextStream in(&file);
	in.setCodec("UTF-8");
	while (!in.atEnd()) {
		QString line = in.readLine();
		if( ! line.startsWith("#") ){
			// First we process the line to replace variables with values :
			line.replace(QString("%IMGDIR%"), QString(ASSUMA_IMGDIR));
			// A line in the file is based on this template :
			// Language name;Language icon;Translation file
			QStringList tmp = line.split(";");
			languageComboBox->addItem( QIcon( tmp.at(1) ), tmp.at(0) );
		}
	}
	
	int lang_idx = languageComboBox->findText( wizardLanguageComboBox->currentText(),  Qt::MatchExactly );
#ifdef ASSUMA_DEBUG
	qDebug() << "Searching for " << wizardLanguageComboBox->currentText() << " in the languageComboBox idx=" << lang_idx;
#endif
	if( lang_idx > -1 )
		languageComboBox->setCurrentIndex(lang_idx);
}

bool AssumaWizard::checkDataDir( const QString & _dataDir ){
	// we want to check for 4 files/directory : langpack.lst, Addons/, PostalCodes/ and Federations
	progressBar->setMaximum(3);
	QStringList errors;
	if(QFile::exists( _dataDir+"/langpack.lst" ) )
		progressBar->setValue(1);
	else
		errors << _dataDir+"/langpack.lst";
	QDir dir(_dataDir+"/Addons");
	if( dir.exists() )
		progressBar->setValue(2);
	else
		errors << _dataDir+"/Addons";
	dir = _dataDir+"/Federations";
	if( dir.exists() )
		progressBar->setValue(3);
	else
		errors << _dataDir+"/Federations";
	
	dir = _dataDir+"/PostalCodes";
	if( dir.exists() )
		progressBar->setValue(4);
	else
		errors << _dataDir+"/PostalCodes";
	
	if( errors.isEmpty() ) {
		return true;
	}
	else{
		errorLabel->setText( "<html>"+tr("The following files/directories could not be found :")+"<br>"+errors.join("<br>\n")+"<br><b>"+tr("This means the data directory is not a valid one for Association Subscribers Manager.")+"</b></html>" );
		return false;
	}
}

void AssumaWizard::pageChanged(int page_number){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaWizard::pageChanged -> page number:" << page_number << " previous page:" << previousPage;
#endif
	QString successMessage("<html><br><b>"+tr("The data directory is valid. Click on \"Next\" to continue.")+"</b></html>");
	if( page_number == 2 ){
		button( QWizard::NextButton )->setEnabled(false);
		//in case we need to redo the checking.
		progressBar->setValue(0);
		errorLabel->setText("");
		if( copyDataDirCheckBox->checkState() == Qt::Checked ){
			errorLabel->setText( "<html><br><b>"+tr("Please wait during file copy.")+"</b></html>" );
			progressBar->setMaximum( QDir( dataDirectoryLineEdit->text()+"/Federations" ).entryList().size() + QDir( dataDirectoryLineEdit->text()+"/PostalCodes" ).entryList().size() + 4 );
			if( checkDataDir( dataDirectoryLineEdit->text() ) ){
				QDir destDir( copyDataDirectoryLineEdit->text() );
				if( destDir.entryList( QDir::AllEntries | QDir::NoDotAndDotDot ).size() > 0 ){
					if( ! destDir.exists("AssociationSubscribersManagerData") )
						destDir.mkdir("AssociationSubscribersManagerData");
					if( ! destDir.cd("AssociationSubscribersManagerData") ){
						errorLabel->setText( "<html><br><font color=\"red\"><b>"+tr("An error occured during the copy. We cannot continue.")+"</b></font></html>" );
					}
				}
				QFile::copy( dataDirectoryLineEdit->text()+"/langpack.lst", destDir.absolutePath()+"/langpack.lst" );
				progressBar->setValue( progressBar->value()+1 );
				destDir.mkdir("Federations");
				progressBar->setValue( progressBar->value()+1 );
				destDir.mkdir("Addons");
				progressBar->setValue( progressBar->value()+1 );
				foreach( QString file, QDir( dataDirectoryLineEdit->text()+"/Federations" ).entryList(QDir::AllEntries | QDir::NoDotAndDotDot) ){
#ifdef ASSUMA_DEBUG
					qDebug() << "==> copy " << file;
#endif
					QFile::copy(dataDirectoryLineEdit->text()+"/Federations/"+file,destDir.absolutePath()+"/Federations/"+file);
					progressBar->setValue( progressBar->value()+1 );
				}
				destDir.mkdir("Addons");
				progressBar->setValue( progressBar->value()+1 );
				QDir _tmpDir( destDir.absolutePath()+"/Addons/" );
				foreach( QString dir, QDir( dataDirectoryLineEdit->text()+"/Addons" ).entryList(QDir::AllEntries | QDir::NoDotAndDotDot) ){
					_tmpDir.mkdir( dir );
#ifdef ASSUMA_DEBUG
					qDebug() << "==> copy " << dataDirectoryLineEdit->text() << "/Addons/" << dir;
#endif
					foreach( QString file, QDir( dataDirectoryLineEdit->text()+"/Addons/"+dir ).entryList(QDir::AllEntries | QDir::NoDotAndDotDot) ){
						QFile::copy(dataDirectoryLineEdit->text()+"/Addons/"+dir+"/"+file,destDir.absolutePath()+"/Addons/"+dir+"/"+file);
					}
				}
				destDir.mkdir("PostalCodes");
				progressBar->setValue( progressBar->value()+1 );
				foreach( QString file, QDir( dataDirectoryLineEdit->text()+"/PostalCodes" ).entryList(QDir::AllEntries | QDir::NoDotAndDotDot) ){
#ifdef ASSUMA_DEBUG
					qDebug() << "==> copy " << file;
#endif
					QFile::copy(dataDirectoryLineEdit->text()+"/PostalCodes/"+file,destDir.absolutePath()+"/PostalCodes/"+file);
					progressBar->setValue( progressBar->value()+1 );
				}
				if( checkDataDir( destDir.absolutePath() ) ){
					dataDirectoryLineEdit->setText( destDir.absolutePath() );
					copyDataDirCheckBox->setCheckState( Qt::Unchecked );
					button( QWizard::NextButton )->setEnabled(true);
					loadLanguageList( dataDirectoryLineEdit->text()+"/langpack.lst" );
					if( previousPage != 3 )
						errorLabel->setText( successMessage );
					loadLanguageList( dataDirectoryLineEdit->text().append("/langpack.lst") );
				}
			}
			else
				errorLabel->setText( "<html><br><font color=\"red\"><b>"+tr("The source data directory (%1) is not valid. We cannot copy it and we cannot continue.").arg( dataDirectoryLineEdit->text() )+"</b></font></html>" );
		}
		else{
			if( checkDataDir( dataDirectoryLineEdit->text() ) ){
				button( QWizard::NextButton )->setEnabled(true);
				loadLanguageList( dataDirectoryLineEdit->text()+"/langpack.lst" );
				if( previousPage != 3 )
					errorLabel->setText( successMessage );
				loadLanguageList( dataDirectoryLineEdit->text().append("/langpack.lst") );
			}
		}
	}
	previousPage = page_number;
}

void AssumaWizard::on_wizardLanguageComboBox_currentIndexChanged(int idx){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaWizard::on_wizardLanguageComboBox_currentIndexChanged(int idx) idx="<<idx<<"";
#endif
	qApp->removeTranslator(m_translator);
	m_translator = new QTranslator;
	if( idx == 1 )
		m_translator->load(":/i18n/assuma_fr-fr.qm");
	else
		m_translator->load(":/i18n/assuma_en-en.qm");
	qApp->installTranslator(m_translator);
	retranslateUi(this);
	setButtonText ( QWizard::BackButton, tr("< &Back") );
	setButtonText ( QWizard::NextButton, tr("&Next >") );
	setButtonText ( QWizard::FinishButton, tr("&Finish") );
	setButtonText ( QWizard::CancelButton, tr("&Cancel") );
	setButtonText ( QWizard::HelpButton, tr("&Help") );
}

void AssumaWizard::on_dataDirectoryToolButton_clicked(){
	QString str = QFileDialog::getExistingDirectory ( this, tr("Select a directory") );
	if( !str.isEmpty() )
		dataDirectoryLineEdit->setText(str);
}

void AssumaWizard::on_copyDataDirectoryToolButton_clicked(){
	QString str = QFileDialog::getExistingDirectory ( this, tr("Select a directory") );
	if( !str.isEmpty() )
		copyDataDirectoryLineEdit->setText(str);
}

void AssumaWizard::showHelp(){
	QUrl url;
	if( currentId() == 0 )
		url.setUrl("qrc:/doc/user/assuma_wizard_doc.html#title_anchor_03");
	else if( currentId() == 1 )
		url.setUrl("qrc:/doc/user/assuma_wizard_doc.html#title_anchor_04");
	else if( currentId() == 2 )
		url.setUrl("qrc:/doc/user/assuma_wizard_doc.html#title_anchor_05");
	else if( currentId() == 3 )
		url.setUrl("qrc:/doc/user/assuma_wizard_doc.html#title_anchor_06");
	else if( currentId() == 4 )
		url.setUrl("qrc:/doc/user/assuma_wizard_doc.html#title_anchor_07");
	else if( currentId() == 5 )
		url.setUrl("qrc:/doc/user/assuma_wizard_doc.html#title_anchor_08");
	
	helpBrowser()->setUrl( url );
	helpBrowser()->show();
}

AssumaHelpBrowser * AssumaWizard::helpBrowser(){
	if( ! m_helpBrowser ){
		m_helpBrowser = new AssumaHelpBrowser(this);
	}
	return m_helpBrowser;
}